(this.webpackJsonpstreamlit_cookie_manager=this.webpackJsonpstreamlit_cookie_manager||[]).push([[0],{5:function(e,n,o){"use strict";o.r(n);var a=o(2),t=(window.parent||window).document,c=null;a.a.events.addEventListener(a.a.RENDER_EVENT,(function(e){var n,o=e.detail;n=o.args.queue,Object.keys(n).forEach((function(e){var o=n[e];if(null===o.value)t.cookie="".concat(encodeURIComponent(e),"=; max-age=0; path=").concat(encodeURIComponent(o.path));else{var a=new Date(o.expires_at);t.cookie="".concat(encodeURIComponent(e),"=").concat(encodeURIComponent(o.value),";")+" expires=".concat(a.toUTCString(),";")+" path=".concat(encodeURIComponent(o.path),";")}}));var i=t.cookie;c===i||o.args.saveOnly||(a.a.setComponentValue(i),c=i)})),a.a.setComponentReady(),a.a.setFrameHeight(0)}},[[5,1,2]]]);
//# sourceMappingURL=main.666d1b34.chunk.js.map